﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Controls;
using System.Windows.Shapes;
using System.Windows.Media;

namespace Amöben
{
    abstract class Lebewesen
    {
        int x, y;
        public int X { get { return x; } }
        public int Y { get { return y; } }
        protected static Random zufall = new Random();
        public Lebewesen()
        {
            x = zufall.Next(0, 300);
            y = zufall.Next(0, 200);
        }
        public Lebewesen(Lebewesen leb)
        {
            x = leb.X;
            y = leb.Y;
        }
        public void Bewege()
        {
            x += zufall.Next(-1, 2);
            y += zufall.Next(-1, 2);
        }
        public abstract void Zeichne(Canvas dieCanvas);
        public abstract List<Lebewesen> Vermehre();

        // Rückgabe: Liste der Gefressenen
        public abstract List<Lebewesen> Friss(List<Lebewesen> z);
    }

    class Amöbe : Lebewesen
    {
        class Mitochondrium // nur Demo für innere Klasse
        {
            int x, y;
        }
        List<Mitochondrium> energie = new List<Mitochondrium>();
        public Amöbe()
        {
        }
        public Amöbe(Amöbe a)
            : base(a)
        {
        }
        int zahlGefressenerBakterien; 
        public override void Zeichne(Canvas dieCanvas)
        {
            Ellipse ellipse = new Ellipse();
            ellipse.Width = 5.0;
            ellipse.Height = 5.0;
            ellipse.Fill = (zahlGefressenerBakterien > 0) ? Brushes.Blue : Brushes.Green;
            dieCanvas.Children.Add(ellipse);
            Canvas.SetLeft(ellipse, X - 2.5);
            Canvas.SetTop(ellipse, Y - 2.5);
        }
        public override List<Lebewesen> Vermehre()
        {
            List<Lebewesen> töchter = new List<Lebewesen>();

            if (zufall.NextDouble() < 0.01 * zahlGefressenerBakterien)
            {
                töchter.Add(new Amöbe(this));
            }

            return töchter;
        }
        public override List<Lebewesen> Friss(List<Lebewesen> z)
        {
            List<Lebewesen> gefressen = new List<Lebewesen>();
            foreach (Lebewesen item in z)
            {
                if (item is Bakterium)
                {
                    int dx = item.X - X;
                    int dy = item.Y - Y;
                    if (dx * dx + dy * dy <= 2 * 2)
                    {
                        gefressen.Add(item);
                        zahlGefressenerBakterien++;
                    }
                }
            }
            return gefressen;
        }
    }

    class Bakterium : Lebewesen
    {
        public override void Zeichne(Canvas dieCanvas)
        {
            Ellipse ellipse = new Ellipse();
            ellipse.Width = 2.0;
            ellipse.Height = 2.0;
            ellipse.Fill = Brushes.Red;
            dieCanvas.Children.Add(ellipse);
            Canvas.SetLeft(ellipse, X - 1.0);
            Canvas.SetTop(ellipse, Y - 1.0);
        }
        public override List<Lebewesen> Vermehre()
        {
            List<Lebewesen> töchter = new List<Lebewesen>();
            if (zufall.NextDouble() < 0.02)
            {
                töchter.Add(new Bakterium(this));
            }
            return töchter;
        }
        public Bakterium()
        {
        }
        public Bakterium(Bakterium a)
            : base(a)
        {
        }
        public override List<Lebewesen> Friss(List<Lebewesen> z)
        {
            return new List<Lebewesen>();
        }
    }
}
